// ApplicationIo.h
//
// Board-specific data flow and hardware I/O

#ifndef ApplicationIoH
#define ApplicationIoH

#include <ProcessEvents_Mb.h>
#include <PacketStream_Mb.h>
#include <StopWatch_Mb.h>
#include <DataLogger_Mb.h>
#include <IniFile_Mb.h>
#include <BinView_Mb.h>
#include "ModuleIo.h"
#include <Script_Mb.h>
#include <Benchmark_Mb.h>
#include <SoftwareTimer_Mb.h>
#include <Application/IniSaver_App.h>
#include <Application/BinviewPlotter_App.h>
#include <Application/FiclIo_App.h>
#include <Exception_Mb.h>
#include <fstream>

class ApplicationIo;

//==============================================================================
//  CLASS ApplicationSettings
//==============================================================================

class ApplicationSettings : public Innovative::IniSaver
{
public:
//    typedef std::vector<char>   BoolArray;
//    typedef std::vector<int>    IntArray;
//    typedef std::vector<float>  FloatArray;

    // Ctor
    ApplicationSettings();
    ~ApplicationSettings();

    // Data
    unsigned int    SamplesToLog;
    unsigned int    PacketSize;
    Innovative::IniFile::CharArray       ActiveChannels;
    std::string     ExoFile;
    std::string     ModuleName;
    std::string     ModuleRevision;
    int             Target;
    int             BusmasterSize;

    int             SampleClockSource;
    int             ReferenceSource;
    float           ReferenceFrequency;
    float           SampleRate;
    int             Divider;

    bool            ExternalTrigger;
    bool            FrontPanelTrigger;

    int             Framed;
    int             FrameSize;
    bool            AutoTrigger;
    std::string     DebugScript;
    double          PhaseDetectorFrequency;
    std::string     StartupScript;
    bool            LoggerEnable;
    bool            PlotEnable;
    bool            AutoStop;
    bool            InitDdcs;
    bool            OverwriteBdd;
    bool            RTPlotEnable;
    float           RTPlotFrequency;
    int             RTPlotSamples;
    int             DioConfig;
    int             FrontPanelConfig;
    bool            TestCounterEnable;
    std::vector<Innovative::IniFile::FloatArray >  Gain;
    std::vector<Innovative::IniFile::FloatArray >  Offset;
    float           PllCorrection;
    bool            PllCalibrated;
    bool            Calibrated;
    Innovative::IniFile::CharArray  AlertEnable;
    bool            Help;
    bool            PretriggerEnable;
    unsigned int    PretriggerCount;
    unsigned int    TriggerDelay;
    int             SyncMaster;
    bool            FilterCoefficientsEnable;
    std::string     FilterCoefficients;
    std::string     AssScript;
    std::string     BssScript;

    Innovative::IniFile::IntArray   Range;

    std::string     Root()
        {  return root;  }

private:
    // Data
    std::string             root;
};

//===========================================================================
//  CLASS IUserInteface  -- Interface for callbacks to UI from App Logic
//===========================================================================

class IUserInterface : public IFicl_UserInterface
{
public:
    virtual ~IUserInterface(){ }
    virtual void  UpdateLogicLoadProgress(int percent) = 0;
    virtual void  Log(const std::string & a_string) = 0;
    virtual void  LogicStatus(const std::string & a_string) = 0;
    virtual void  Status(const std::string & a_string) = 0;
    virtual void  GetSettings() = 0;
    virtual void  SetSettings() = 0;
    virtual void  AfterStreamAutoStop() = 0;
    virtual void  Warning(const std::string & a_string) = 0;
    virtual void  PeriodicStatus() = 0;
    virtual void  DisplayHelp() = 0;
};

//===========================================================================
//  CLASS ApplicationIo  -- Hardware Access and Application Io Class
//===========================================================================

class ApplicationIo : public FiclIo
{
    friend class ApplicationSettings;
    typedef std::vector<__int64>    IntArray;

public:
    //
    //  Member Functions
    ApplicationIo(IUserInterface * ui);
    ~ApplicationIo();

    void  FiclInit();

    ModuleIo &  ModIo()
        {  return Module;  }

    unsigned int    BoardCount();
    vector<string>  BoardNames();
    unsigned int    SetTargetIndex();
    string          PrintDevices();
    void   Open();
    bool   IsOpen()
        {  return Opened;  }
    void   Close();
    void   LoadLogic();
    void   StartStreaming();
    void   StopStreaming();
    bool   IsStreaming()
        {  return Timer.Enabled();  }
    bool   IsTriggered()
        {  return FTriggered >= 0;  }
    void   TestZbt();
    void   WriteRom();
    void   ReadRom();
    void   ExecuteDebugScript();
    void   SoftwareAlert(unsigned int value);
    float  Temperature();
    float  SampleRate() const
        {  return static_cast<float>(Settings.SampleRate*1.e3);  }
    float  ReferenceRate() const
        {  return static_cast<float>(Settings.ReferenceFrequency*1.e6);  }

double BlockRate() const
        {  return FBlockRate;  }
    std::string Version()
        {  return FVersion;  }
    unsigned int Channels() const
        {  return static_cast<int>(AnalogInChannels());  }
    ii64	BlockCount() const
        {  return FBlockCount;  }
    unsigned int Ranges() const
        {  return static_cast<int>(AnalogInRanges());  }

    std::string     ModuleName() const;
    std::string     ModuleRevision();

    void            DioData(unsigned int value);
    unsigned int    DioData();
    void            FrontPanelData(unsigned int value);
    unsigned int    FrontPanelData();
    void            CalculateExternalFrequency();
    bool            Pll_Locked();

    bool            PretriggerSupported();
    bool            TriggerDelaySupported();

    bool            LogicLoadCompleted() const
      {  return FLogicLoaded;  }
    bool            LogicLoadError() const
      {  return FLogicError;  }

    void    ExecuteDebugScript(const std::string & cmd)
        {  Execute(std::string("load ") + cmd);  }
    void    Help()
        {  Execute(Module.Help());  }
    unsigned int Peek(unsigned int addr)
        {  return Module().PeekLogic(addr);    }

    //version
    std::string SoftwareVersionString()
    { return Module.SoftwareVersionString(); }


    // Data
    ApplicationSettings     Settings;

    unsigned int            BBoardCount;

private:
    //
    //  Member Data
    ModuleIo                 			Module;
    IUserInterface *                    UI;
    Innovative::PacketStream            Stream;
    Innovative::PacketImageParser       Pip;
    unsigned int                        Cursor;
    ii64                                BlocksToLog;

    bool                                Opened;
    bool                                Stopped;
    bool                                StreamConnected;
    Innovative::StopWatch               Clock;
    Innovative::DataLogger              Logger;
    IntArray                            DataRead;
    Innovative::BinView                 Graph;
    float                               ActualSampleRate;
    std::string                         Root;
    Innovative::AveragedRate            Time;
    Innovative::AveragedRate            BytesPerBlock;
    double                              FBlockRate;
    std::string                         FVersion;
    Innovative::SoftwareTimer           Timer;
    ii64                                FBlockCount;
    int                                 FDevices;
    int                                 FTicks;
    int                                 FTriggered;
    Innovative::BinviewPlotter          Plot;

    size_t                              TriggerTime;
    size_t                              StopTriggerTime;
    Innovative::DataLogger              TriggerLogger;
    Innovative::DataLogger              StopTriggerLogger;
    bool                                FLogicLoaded;
    bool                                FLogicError;

    //
    //  Member Functions
    void  HandleProgress(Innovative::ProcessProgressEvent & event);
    void  HandleParseComplete(Innovative::ProcessCompletionEvent & event);
    void  HandleLoadComplete(Innovative::ProcessCompletionEvent & event);
    void  HandleLoadError(Innovative::ProcessStatusEvent & event);

    void  HandleDataAvailable(Innovative::PacketStreamDataEvent & Event);
    void  HandleDirectDataAvailable(Innovative::PacketStreamDirectDataEvent & Event);
    void  HandlePipProcessing(Innovative::PacketImageParserImageAvailable & Event);
    void  HandleTimestampRolloverAlert(Innovative::AlertSignalEvent & event);
    void  HandleSoftwareAlert(Innovative::AlertSignalEvent & event);
    void  HandleWarningTempAlert(Innovative::AlertSignalEvent & event);
    void  HandlePllLostAlert(Innovative::AlertSignalEvent & event);
    void  HandleInputFifoOverrunAlert(Innovative::AlertSignalEvent & event);
    void  HandleInputTriggerAlert(Innovative::AlertSignalEvent & event);
    void  HandleInputStopTriggerAlert(Innovative::AlertSignalEvent & event);
    void  HandleInputPretriggerErrorAlert(Innovative::AlertSignalEvent & event);
    void  HandleInputFifoOverrangeAlert(Innovative::AlertSignalEvent & event);

    void  HandleBeforeStreamStart(OpenWire::NotifyEvent & Event);
    void  HandleAfterStreamStart(OpenWire::NotifyEvent & Event);
    void  HandleAfterStreamStop(OpenWire::NotifyEvent & Event);
    void  HandleScriptCommand(Innovative::ProcessStatusEvent & Event);
    void  HandleScriptMessage(Innovative::ProcessStatusEvent & Event);
    void  HandleTimer(OpenWire::NotifyEvent & Event);

    void  HandleOnLog(Innovative::ProcessStatusEvent & Event);

    void  Log(const std::string & msg);
    void  Log(const std::string & msg, double argument);
    void  Log(const std::string & msg, unsigned int argument);
    OpenWire::ThunkedEventHandler<Innovative::ProcessStatusEvent>  OnLog;

    void  DoLogicStore();
    void  DoLogicFetch();
    void  DoPortStore();
    void  DoPortFetch();
    void  DoDelay();

    bool  IsDataLoggingCompleted();
    void  InitBddFile();
    void  DisplayLogicVersion();
    void  SoftwareTrigger();
    std::string Elapsed(size_t timestamp);
    void  TallyBlock(size_t bytes);
};

#endif
