// ApplicationIo.h
//
//  Board-specific software layer
//
//

#ifndef ApplicationIoH
#define ApplicationIoH

#include <ProcessEvents_Mb.h>
#include <PacketStream_Mb.h>
#include <Buffer_Mb.h>                 
#include <StopWatch_Mb.h>
#include "Target.h"
#include <Script_Mb.h>
#include <SoftwareTimer_Mb.h>
#include <IniFile_Mb.h>
#include <Benchmark_Mb.h>
#include <Exception_Mb.h>

class ApplicationIo;

//==============================================================================
//  CLASS ApplicationSettings
//==============================================================================

class ApplicationSettings
{
public:
    typedef std::vector<char>   BoolArray;
    typedef std::vector<int>    IntArray;
    typedef std::vector<float>  FloatArray;

    // Ctor
    ApplicationSettings(int channels);
    ~ApplicationSettings();

    // Data
    BoolArray       ActiveChannels;
    std::string     ModuleName;
    std::string     ModuleRevision;
	std::string     BssScript;
    std::string     AssScript;
    std::string     DebugScript;
    double          PhaseDetectorFrequency;
    std::string     WaveFile;
    int             Target;
    int             SampleClockSource;
	int             ReferenceSource;
	float           ReferenceFrequency;
    int             BusmasterSize;
    int             ExternalTrigger;
    int             Framed;
    int             FrameCount;
    int             DecimationFactor;
    int             StreamPacketSize;
    int             WaveType;
	float 			WaveformFrequency;
	float 			WaveformAmplitude;
    bool            DecimationEnable;
    bool            AutoTrigger;
    bool            Calibrated;
	float           PllCorrection;
    bool            PllCalibrated;
    float           SampleRate;
	int             PrefillPeriod;
	std::string     ExoFile;
    int             DioConfig;
	int             FrontPanelConfig;
	bool			TestModeEnable;
	bool			FastClock;

    BoolArray                           ScriptEnable;
    Innovative::IniFile::FloatArray     DacGain;
    Innovative::IniFile::FloatArray     DacOffset;
    BoolArray                           AlertEnable;

    std::string     Root()
        {  return root;  }

private:
    // Data
    Innovative::IniFile *   Ini;
    std::string             root;
};


//===========================================================================
//  CLASS IUserInteface  -- Interface for callbacks to UI from App Logic
//===========================================================================

class IUserInterface
{
public:
    virtual ~IUserInterface()  {  }
    virtual void  UpdateLogicLoadProgress(int percent) = 0;
    virtual void  Log(const std::string & a_string) = 0;
    virtual void  Status(const std::string & a_string) = 0;
    virtual void  GetSettings() = 0;
    virtual void  SetSettings() = 0;
    virtual void  AfterStreamStop() = 0;
    virtual void  Warning(const std::string & a_string) = 0;
    virtual void  ClockFrequency(float value) = 0;
    virtual void  PeriodicStatus() = 0;
    virtual void  LogicVersionStatus(const std::string & txt) = 0;
};


//==============================================================================
//  CLASS ApplicationIo
//==============================================================================

class ApplicationIo
{
public:
    // Ctor
    ApplicationIo(IUserInterface * gui);
    ~ApplicationIo();

    // Methods
    void            Open();
    void            Close();
    void            Start();
	bool            StartStreaming();
    void            StopStreaming();
    void            Stop();
	void            LoadLogic();
    void            WriteRom();
    void            ReadRom();
    bool            IsOpen();
    bool            IsTriggered();
    bool            IsDataCheckError();
	bool            IsStreaming();
	bool            IsPllLocked();
    float           Temperature();
    float           SampleRate() const;
    float           ReferenceRate() const;
    double          BlockRate() const;
    int             Channels() const;
    int             BlockCount() const;
    int             BoardCount();
    vector<string>  BoardNames();
    unsigned int    SetTargetIndex();
    string          PrintDevices();
    void            DioData(unsigned int value);
    unsigned int    DioData();
    void            FrontPanelData(unsigned int value);
    unsigned int    FrontPanelData();

    void   SoftwareAlert(unsigned int value);
 
    void ExecuteDebugScript();
    void PeekPort(int address);
    void PokePort(int address, int data);

    void PokeLogic(int address, int data);
    void PeekLogic(int address);

    void SpiDump();
    std::string Root()
        {  return Settings.Root();  }

    //version
    std::string SoftwareVersionString()
    { return std::string("2.0.0"); }

    // Data
    ApplicationSettings     Settings;

    unsigned int            BBoardCount;

protected:
    enum IIState { stIdle, stGenerating, stStreaming };

	PmcModule                       Module;
	Innovative::PacketStream        Stream;
	IUserInterface           *      UI;
	Innovative::Scripter            Script;
	Innovative:: Buffer   *         Packet;
    Innovative::AveragedRate        Time;
    Innovative::SoftwareTimer       Timer;
    double                          FBlockRate;
    bool                            FOpened;
    bool                            FStreaming;
    bool                            FStreamConnected;
    int                             FTriggered;
    int                             FTicks;
    int                             FBlockCount;
    int                             FChannels;
    int                             FDevices;
	unsigned int                    FBusmasterSize;
    short                           TestCounter;
    int                             PrefillCount;
    int                             PrefillPacketCount;
    Innovative::Buffer              WaveformPacket;


    void  HandleSoftwareAlert(Innovative::AlertSignalEvent & event);
    void  HandleTimestampRolloverAlert(Innovative::AlertSignalEvent & event);
    void  HandleWarningTempAlert(Innovative::AlertSignalEvent & event);
    void  HandleOutputFifoUnderrunAlert(Innovative::AlertSignalEvent & event);
    void  HandleOutputTriggerAlert(Innovative::AlertSignalEvent & event);
    void  HandlePllLostAlert(Innovative::AlertSignalEvent & event);
    
    void  HandleDataRequired(Innovative::PacketStreamDataEvent & Event);
    void  HandleBeforeStreamStart(OpenWire::NotifyEvent & Event);
    void  HandleAfterStreamStart(OpenWire::NotifyEvent & Event);
    void  HandleAfterStreamStop(OpenWire::NotifyEvent & Event);
    void  HandleOnLog(Innovative::ProcessStatusEvent & Event);

    void  HandleScriptCommand(Innovative::ProcessStatusEvent & Event);
	void  HandleScriptMessage(Innovative::ProcessStatusEvent & Event);
    void  HandleTimer(OpenWire::NotifyEvent & Event);

	void  HandleProgress(Innovative::ProcessProgressEvent & event);
	void  HandleParseComplete(Innovative::ProcessCompletionEvent & event);
	void  HandleLoadComplete(Innovative::ProcessCompletionEvent & event);
	void  HandleLoadError(Innovative::ProcessStatusEvent & event);

    void  SendOneBlock(Innovative::PacketStream * PS);
	void  BuildWave(Innovative::Buffer & Buffer, int WaveType);

    void  Log(const std::string & msg);
    OpenWire::ThunkedEventHandler<Innovative::ProcessStatusEvent>  OnLog;

    void  DoLogicStore();
    void  DoLogicFetch();
    void  DoPortStore();
    void  DoPortFetch();
    void  DoDelay();

    void  DisplayLogicVersion();
    void  SoftwareTrigger();
    std::string Elapsed(size_t timestamp);
};
#endif
