#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Malibu_Mb.h>
#include <StringSupport_Mb.h>
#include <MalibuVersion_Mb.h>

using namespace Innovative;
using namespace std;


//===========================================================================
//  CLASS MainWindow  -- GUI Form
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::MainWindow() --  Ctor
//---------------------------------------------------------------------------

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    UI = new UserInterface(this);
    Io = new ApplicationIo(UI);


    //Information about the imgae
    ToControl(ui->ConfigSwVersionLineEdit, Io->SoftwareVersionString());
    ToControl(ui->ConfigSwBuildDateLineEdit, std::string(__DATE__));
    ToControl(ui->ConfigSwBuildTimeLineEdit, std::string(__TIME__));
    ToControl(ui->ConfigMalibuVersionLineEdit, MalibuVersion().Malibu());

    // Format channels check list box
    for (int i = 0; i < Io->Channels(); ++i)
        {
        std::stringstream tag;
        tag << "Ch " << i;
        AddToControl(ui->ActiveChannelsCheckListBox, tag.str());
        }

    // Format coefficients grid
    SetDimensions(ui->DacCoefGrid, 2, Io->Channels());

    for (int i = 0; i < Io->Channels(); ++i)
        ToRowHeading(ui->DacCoefGrid, i, std::string("Ch") + IntToString(i));

    ToColumnHeading(ui->DacCoefGrid, 0, "Gain");
    ToColumnHeading(ui->DacCoefGrid, 1, "Offset");

    //
    //  Set up Target Listbox
    int Count = Io->BBoardCount = Io->BoardCount();
    if (!Count)
        Log("No devices detected!");

    // Populate target combo box with actual boards detected
    BoardNames = Io->BoardNames();
    FillControl(ui->TargetComboBox, BoardNames);
    ToControl(ui->TargetComboBox, 0);

    SetSettings();

    // Initialize grids, Disable selection region
    ShowTab(ui->ConfigTabSheet, ui->TabSheet, true);

    DioStatusBar = new QStatusBar(ui->DioStatusBarFrame);
    DioStatusBar->setSizeGripEnabled(false);

    PerformanceStatusBar = new QStatusBar(ui->PerformancStatusBarFrame);
    PerformanceStatusBar->setSizeGripEnabled(false);
    PeriodicStatus();

    LogicStatusBar = new QStatusBar(ui->LogicStatusBarFrame);
    LogicStatusBar->setSizeGripEnabled(false);
    LogicStatus("Unknown");

    ui->TargetComboBox->setCurrentIndex(Io->SetTargetIndex());
    Log(Io->PrintDevices());
}

//---------------------------------------------------------------------------
//  MainWindow::~MainWindow() --  Dtor
//---------------------------------------------------------------------------

MainWindow::~MainWindow()
{
   GetSettings();

    delete Io;
    Io = 0;

    delete UI;
    UI = 0;

    delete ui;
}

//---------------------------------------------------------------------------
//  MainWindow::~changeEvent() --
//---------------------------------------------------------------------------

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

//===========================================================================
//  Event Handlers
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::AdjustEnables() --
//---------------------------------------------------------------------------

void MainWindow::AdjustEnables()
{
    ControlEnabled(ui->TargetComboBox, !Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->TargetLabel, !Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->BusmasterSizeComboBox, !Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->BusmasterSizeLabel, !Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->OpenBtn, !Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->CloseBtn, Io->IsOpen());
    ControlEnabled(ui->LogicLoadConfigBtn, Io->IsOpen());
    int idx = ReadControl<int>(ui->ClockSourceComboBox);
    ControlEnabled(ui->ReferenceSourceComboBox, idx==2);
    ControlEnabled(ui->ReferenceSourceLabel, idx==2);
    ControlEnabled(ui->ExternalClockFrequencyEdit, idx!=2);

    ControlEnabled(ui->StreamTabSheet, Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->EepromTabSheet, Io->IsOpen()&&Io->BBoardCount);
    ControlEnabled(ui->DebugTabSheet, Io->IsOpen()&&Io->BBoardCount);
}

//---------------------------------------------------------------------------
//  MainWindow::on_OpenBtn_clicked() --  Open driver
//---------------------------------------------------------------------------

void MainWindow::on_OpenBtn_clicked()
{
    Io->Settings.Target = ReadControl<int>(ui->TargetComboBox);
    if (BoardNames[Io->Settings.Target] != "X3 XMCe")
        {
        QMessageBox::information(this,tr("Wrong Target"), tr("Target should be X3 XMCe. Please check target."));
        return;
        }

    Io->Open();

    AdjustEnables();

    on_LoadFromRomButton_clicked();
}

//---------------------------------------------------------------------------
//  MainWindow::on_CloseBtn_clicked() --  Close driver
//---------------------------------------------------------------------------

void MainWindow::on_CloseBtn_clicked()
{
    Io->Close();

    AdjustEnables();
}

//---------------------------------------------------------------------------
//  MainWindow::on_StreamStartBtn_clicked() --  Start stream
//---------------------------------------------------------------------------

void MainWindow::on_StreamStartBtn_clicked()
{
    ClearControl(ui->MessageLog);
    //
    //  Activate stop button, deactivate start btn
    ControlEnabled(ui->StreamStartBtn, false);
    ControlEnabled(ui->StreamStopBtn, true);

    Io->StartStreaming();
}

//---------------------------------------------------------------------------
//  MainWindow::on_StreamStopBtn_clicked() --  Stop stream
//---------------------------------------------------------------------------

void MainWindow::on_StreamStopBtn_clicked()
{
    //
    //  Deectivate stop button, activate start btn
    ControlEnabled(ui->StreamStartBtn, true);
    ControlEnabled(ui->StreamStopBtn, false);

    Io->StopStreaming();
}

//---------------------------------------------------------------------------
//  MainWindow::on_LoadFromRomButton_clicked() --  Load ROM
//---------------------------------------------------------------------------

void MainWindow::on_LoadFromRomButton_clicked()
{
    Io->ReadRom();

    for (int i = 0; i < Io->Channels(); ++i)
        {
        ToControl(ui->DacCoefGrid, 0, i, FloatToString(Io->Settings.DacGain[i], 4));
        ToControl(ui->DacCoefGrid, 1, i, FloatToString(Io->Settings.DacOffset[i], 4));
        }

    ToControl(ui->CalibratedCheckBox, Io->Settings.Calibrated);
    ToControl(ui->ModuleNameEdit, Io->Settings.ModuleName);
    ToControl(ui->ModuleRevisionEdit, Io->Settings.ModuleRevision);
 }

//---------------------------------------------------------------------------
//  MainWindow::on_StoreToRomButton_clicked() --  Load ROM
//---------------------------------------------------------------------------

void MainWindow::on_StoreToRomButton_clicked()
{
    for (int i = 0; i < Io->Channels(); ++i)
        {
        FromControl(ui->DacCoefGrid, 0, i, Io->Settings.DacGain[i]);
        FromControl(ui->DacCoefGrid, 1, i, Io->Settings.DacOffset[i]);
        }

    FromControl(ui->CalibratedCheckBox, Io->Settings.Calibrated);
    FromControl(ui->ModuleNameEdit, Io->Settings.ModuleName);
    FromControl(ui->ModuleRevisionEdit, Io->Settings.ModuleRevision);

    Io->WriteRom();
}

//---------------------------------------------------------------------------
//  MainWindow::on_DebugScriptBrowseButton_clicked() --  Select debug script
//---------------------------------------------------------------------------

void MainWindow::on_DebugScriptBrowseButton_clicked()
{
    SelectFileDialog(ui->DebugScriptEdit, "Script Files (*.fr *.*");
}

//---------------------------------------------------------------------------
//  MainWindow::on_DebugScriptBrowseButton_clicked() --  Execute debug script
//---------------------------------------------------------------------------

void MainWindow::on_DebugScriptExecuteButton_clicked()
{
//    Io->ExecuteDebugScript(ReadControl<std::string>(ui->DebugScriptEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_SoftwareAlertButton_clicked() --  Execute software alert
//---------------------------------------------------------------------------

void MainWindow::on_SoftwareAlertButton_clicked()
{
    Io->SoftwareAlert(ReadControl<int>(ui->SoftwareAlertEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_LogicReadButton_clicked() --  Read logic port
//---------------------------------------------------------------------------

void MainWindow::on_LogicReadButton_clicked()
{
    Io->PeekLogic(ReadControl<int>(ui->AddressEdit));
//    Io->Logic(ReadControl<std::string>(ui->AddressEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_LogicWriteButton_clicked() --  Write logic port
//---------------------------------------------------------------------------

void MainWindow::on_LogicWriteButton_clicked()
{
    Io->PokeLogic(ReadControl<int>(ui->AddressEdit), ReadControl<int>(ui->DataEdit));
 //   Io->Logic(ReadControl<std::string>(ui->AddressEdit), ReadControl<std::string>(ui->DataEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_PortReadButton_clicked() --  Read PCI port
//---------------------------------------------------------------------------

void MainWindow::on_PortReadButton_clicked()
{
    Io->PeekPort(ReadControl<int>(ui->AddressEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_PortWriteButton_clicked() --  Write PCI port
//---------------------------------------------------------------------------

void MainWindow::on_PortWriteButton_clicked()
{
    Io->PokePort(ReadControl<int>(ui->AddressEdit), ReadControl<int>(ui->DataEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_DeviceReadButton_clicked() --  Read device port
//---------------------------------------------------------------------------

void MainWindow::on_DeviceReadButton_clicked()
{
//    Io->Device(ReadControl<std::string>(ui->AddressEdit),
//               ReadControl<std::string>(ui->DeviceEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_DeviceWriteButton_clicked() --  Write device port
//---------------------------------------------------------------------------

void MainWindow::on_DeviceWriteButton_clicked()
{
//    Io->Device(ReadControl<std::string>(ui->AddressEdit),
//               ReadControl<std::string>(ui->DeviceEdit),
//               ReadControl<std::string>(ui->DataEdit));
}

//---------------------------------------------------------------------------
//  MainWindow::on_FiclCommandEdit_returnPressed() --
//---------------------------------------------------------------------------

void MainWindow::on_FiclCommandEdit_returnPressed()
{
//    char Key = 0x0d;
//    Io->FiclIo_OnCommandKeyPress(Key);
}

//---------------------------------------------------------------------------
//  MainWindow::on_FiclCommandEdit_returnPressed() --
//---------------------------------------------------------------------------

void MainWindow::on_LogicLoadBrowseBtn_clicked()
{
    SelectFileDialog(ui->LogicFilenameEdit, "Image File *.bit *.exo *.*");
}

//---------------------------------------------------------------------------
//  MainWindow::on_LogicLoadConfigBtn_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_LogicLoadConfigBtn_clicked()
{
    Io->LoadLogic();
}

//---------------------------------------------------------------------------
//  MainWindow::on_BssButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_BssButton_clicked()
{
    SelectFileDialog(ui->BssScriptEdit, "Script File *.txt *.*");
}

//---------------------------------------------------------------------------
//  MainWindow::on_AssButton_clicked() --
//---------------------------------------------------------------------------

void MainWindow::on_AssButton_clicked()
{
    SelectFileDialog(ui->AssScriptEdit, "Script File *.txt *.*");
}

//===========================================================================
//  Support Methods
//===========================================================================

//---------------------------------------------------------------------------
//  MainWindow::Log() --
//---------------------------------------------------------------------------

void  MainWindow::Log(const std::string & a_string)
{
    AddToControl(ui->MessageLog, a_string);
}

//---------------------------------------------------------------------------
//  MainWindow::UpdateLogicLoadProgress() --
//---------------------------------------------------------------------------

void  MainWindow::UpdateLogicLoadProgress(int percent)
{
    ToControl(ui->LogicLoadProgress, percent);
}


//---------------------------------------------------------------------------
//  MainWindow::Status() --
//---------------------------------------------------------------------------

void  MainWindow::Status(const std::string & a_string)
{
    ToControl(ui->StatusBar, 0, a_string);
}

//---------------------------------------------------------------------------
//  MainWindow::LogicStatus() --
//---------------------------------------------------------------------------

void  MainWindow::LogicStatus(const std::string & a_string)
{
    ToControl(LogicStatusBar, 0, a_string);
}

//---------------------------------------------------------------------------
//  MainWindow::AfterStreamAutoStop() --
//---------------------------------------------------------------------------

void  MainWindow::AfterStreamStop()
{
    //
    //  Deectivate stop button, activate start btn
    ControlEnabled(ui->StreamStartBtn, true);
    ControlEnabled(ui->StreamStopBtn, false);

    // Update the Stream statistics
    PeriodicStatus();
}

//---------------------------------------------------------------------------
//  MainWindow::PeriodicStatus() --
//---------------------------------------------------------------------------

void MainWindow::PeriodicStatus()
{
    ToControl(PerformanceStatusBar, 0, IntToString(Io->BlockCount()), "Block Count: ");
    ToControl(PerformanceStatusBar, 1, FloatToString(Io->BlockRate(), 1), "Rate (KB/s): ");
    ToControl(PerformanceStatusBar, 2, FloatToString(Io->Temperature(), 0), "Temp (C): ");
    ToControl(PerformanceStatusBar, 3, std::string("Trig: ") + std::string(Io->IsTriggered() ? "+" : "-"));
    ToControl(PerformanceStatusBar, 4, std::string("Comm Err: ") + std::string(Io->IsDataCheckError() ? "+" : "-"));
    ToControl(PerformanceStatusBar, 5, std::string("PLL Lock ") + std::string(Io->IsPllLocked() ? "+" : "-"));

    ToControl(DioStatusBar, 0, IntToString(Io->DioData(), 16), "Dig: ");
    ToControl(DioStatusBar, 1, IntToString(Io->FrontPanelData(), 16), "Front: ");
}

//---------------------------------------------------------------------------
//  MainWindow::ClockFrequency() --
//---------------------------------------------------------------------------

void  MainWindow::ClockFrequency(float value)
{
    ToControl(ui->SampleFrequencyEdit, value, 3);
}


//---------------------------------------------------------------------------
//  MainWindow::on_ClockSourceComboBox_currentIndexChanged() --
//---------------------------------------------------------------------------

void MainWindow::on_ClockSourceComboBox_currentIndexChanged(int /*index*/)
{
    AdjustEnables();
}


#if 0
//---------------------------------------------------------------------------
//  MainWindow::FiclInit() --
//---------------------------------------------------------------------------

void  MainWindow::FiclInit()
{
    if (HasFiclSupport()==false)
        return;

    //  Link Module to Ficl System
    Io->ModIo().FiclConnectTo(Io->FiclSys());

    Io->FiclIo_Init();
}

//-----------------------------------------------------------------------------
//  MainWindow::FiclReadCommandString()  --
//-----------------------------------------------------------------------------

std::string MainWindow::FiclReadCommandString()
{
    return ReadControl<std::string>(ui->FiclCommandEdit);
}

//-----------------------------------------------------------------------------
//  MainWindow::FiclCommandComplete()  --
//-----------------------------------------------------------------------------

void MainWindow::FiclCommandComplete()
{
    ClearControl(ui->FiclCommandEdit);
}

#endif

//-----------------------------------------------------------------------------
//  MainWindow::on_FiclCommandEdit_textEdited()  --
//-----------------------------------------------------------------------------

void MainWindow::on_FiclCommandEdit_textEdited(QString )
{
//    Io->FiclIo_OnCommandKeyPress(Key);
}

//---------------------------------------------------------------------------
//  MainWindow::GetSettings() --  Form controls to options object
//---------------------------------------------------------------------------

void MainWindow::GetSettings()
{
    FromControl(ui->PacketSizeEdit, Io->Settings.StreamPacketSize);
    FromControl(ui->FrameCountEdit, Io->Settings.FrameCount);
    FromControl(ui->BssScriptEdit, Io->Settings.BssScript);
    FromControl(ui->AssScriptEdit, Io->Settings.AssScript);
    FromControl(ui->WaveFileEdit, Io->Settings.WaveFile);
    FromControl(ui->DecimationFactorEdit, Io->Settings.DecimationFactor);
    FromControl(ui->DecimationCheckBox, Io->Settings.DecimationEnable);
    FromControl(ui->DioConfigEdit, Io->Settings.DioConfig);
    FromControl(ui->FrontPanelConfigEdit, Io->Settings.FrontPanelConfig);

    FromControl(ui->SampleFrequencyEdit, Io->Settings.SampleRate);
    FromControl(ui->BusmasterSizeComboBox, Io->Settings.BusmasterSize);
    FromControl(ui->TriggerSourceListBox, Io->Settings.ExternalTrigger);
    FromControl(ui->TriggerModeListBox, Io->Settings.Framed);
    FromControl(ui->WaveTypeListBox, Io->Settings.WaveType);
    FromControl(ui->TargetComboBox, Io->Settings.Target);
    FromControl(ui->WaveformFrequencyEdit, Io->Settings.WaveformFrequency);
    FromControl(ui->WaveformAmplitudeEdit, Io->Settings.WaveformAmplitude);

    FromControl(ui->AutoTriggerCheckBox, Io->Settings.AutoTrigger);
    FromControl(ui->BssScriptEnableCheckBox, Io->Settings.ScriptEnable[0]);
    FromControl(ui->AssScriptEnableCheckBox, Io->Settings.ScriptEnable[1]);

    FromControl(ui->DebugScriptEdit, Io->Settings.DebugScript);
    FromControl(ui->LogicFilenameEdit, Io->Settings.ExoFile);

    FromControl(ui->ClockSourceComboBox, Io->Settings.SampleClockSource);
    FromControl(ui->ReferenceSourceComboBox, Io->Settings.ReferenceSource);
    FromControl(ui->ReferenceFrequencyEdit, Io->Settings.ReferenceFrequency);
    FromControl(ui->PrefillPeriodEdit, Io->Settings.PrefillPeriod);

    FromControl(ui->AlertEnableCheckListBox, Io->Settings.AlertEnable);
    FromControl(ui->ActiveChannelsCheckListBox, Io->Settings.ActiveChannels);
}

//---------------------------------------------------------------------------
//  MainWindow::SetSettings() --  Settings object to form controls
//---------------------------------------------------------------------------

void MainWindow::SetSettings()
{
    ToControl(ui->PacketSizeEdit, IntToString(Io->Settings.StreamPacketSize, 16, true));
    ToControl(ui->FrameCountEdit, IntToString(Io->Settings.FrameCount, 16, true));
    ToControl(ui->BssScriptEdit, Io->Settings.BssScript);
    ToControl(ui->AssScriptEdit, Io->Settings.AssScript);
    ToControl(ui->WaveFileEdit, Io->Settings.WaveFile);
    ToControl(ui->DecimationFactorEdit, IntToString(Io->Settings.DecimationFactor, 10));
    ToControl(ui->DecimationCheckBox, Io->Settings.DecimationEnable);
    ToControl(ui->DioConfigEdit, Io->Settings.DioConfig, 16);
    ToControl(ui->FrontPanelConfigEdit, Io->Settings.FrontPanelConfig, 16);

    ToControl(ui->SampleFrequencyEdit, FloatToString(Io->Settings.SampleRate));
    ToControl(ui->BusmasterSizeComboBox, Io->Settings.BusmasterSize);
    ToControl(ui->TriggerSourceListBox, Io->Settings.ExternalTrigger);
    ToControl(ui->TriggerModeListBox, Io->Settings.Framed);
    ToControl(ui->WaveTypeListBox, Io->Settings.WaveType);
    ToControl(ui->TargetComboBox, Io->Settings.Target);
    ToControl(ui->WaveformFrequencyEdit, FloatToString(Io->Settings.WaveformFrequency));
    ToControl(ui->WaveformAmplitudeEdit, FloatToString(Io->Settings.WaveformAmplitude));

    ToControl(ui->AutoTriggerCheckBox, Io->Settings.AutoTrigger);
    ToControl(ui->BssScriptEnableCheckBox, Io->Settings.ScriptEnable[0]);
    ToControl(ui->AssScriptEnableCheckBox, Io->Settings.ScriptEnable[1]);

    ToControl(ui->DebugScriptEdit, Io->Settings.DebugScript);
    ToControl(ui->LogicFilenameEdit, Io->Settings.ExoFile);

    ToControl(ui->ClockSourceComboBox, Io->Settings.SampleClockSource);
    ToControl(ui->ReferenceSourceComboBox, Io->Settings.ReferenceSource);
    ToControl(ui->ReferenceFrequencyEdit, Io->Settings.ReferenceFrequency);
    ToControl(ui->PrefillPeriodEdit, IntToString(Io->Settings.PrefillPeriod));

    ToControl(ui->AlertEnableCheckListBox, Io->Settings.AlertEnable);
    ToControl(ui->ActiveChannelsCheckListBox, Io->Settings.ActiveChannels);
}


